# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/finsets',
                'doc_host': 'https://ionmihai.github.io',
                'git_url': 'https://github.com/ionmihai/finsets',
                'lib_path': 'finsets'},
  'syms': { 'finsets.cli': {'finsets.cli.search': ('cli.html#search', 'finsets/cli.py')},
            'finsets.fetch_tools': { 'finsets.fetch_tools.get_text_file_from_url': ( 'fetch_tools.html#get_text_file_from_url',
                                                                                     'finsets/fetch_tools.py')},
            'finsets.fred.fred': { 'finsets.fred.fred.default_raw_vars': ('00_fred/fred.html#default_raw_vars', 'finsets/fred/fred.py'),
                                   'finsets.fred.fred.get_raw_data': ('00_fred/fred.html#get_raw_data', 'finsets/fred/fred.py'),
                                   'finsets.fred.fred.get_series_info': ('00_fred/fred.html#get_series_info', 'finsets/fred/fred.py'),
                                   'finsets.fred.fred.parse_varlist': ('00_fred/fred.html#parse_varlist', 'finsets/fred/fred.py'),
                                   'finsets.fred.fred.process_raw_data': ('00_fred/fred.html#process_raw_data', 'finsets/fred/fred.py'),
                                   'finsets.fred.fred.search': ('00_fred/fred.html#search', 'finsets/fred/fred.py')},
            'finsets.fred.fred_api': { 'finsets.fred.fred_api.Fred': ('00_fred/fred_api.html#fred', 'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred.__do_series_search': ( '00_fred/fred_api.html#fred.__do_series_search',
                                                                                          'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred.__fetch_data': ( '00_fred/fred_api.html#fred.__fetch_data',
                                                                                    'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred.__get_search_results': ( '00_fred/fred_api.html#fred.__get_search_results',
                                                                                            'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred.__init__': ( '00_fred/fred_api.html#fred.__init__',
                                                                                'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred._parse': ( '00_fred/fred_api.html#fred._parse',
                                                                              'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred.get_series': ( '00_fred/fred_api.html#fred.get_series',
                                                                                  'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred.get_series_all_releases': ( '00_fred/fred_api.html#fred.get_series_all_releases',
                                                                                               'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred.get_series_as_of_date': ( '00_fred/fred_api.html#fred.get_series_as_of_date',
                                                                                             'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred.get_series_first_release': ( '00_fred/fred_api.html#fred.get_series_first_release',
                                                                                                'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred.get_series_info': ( '00_fred/fred_api.html#fred.get_series_info',
                                                                                       'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred.get_series_latest_release': ( '00_fred/fred_api.html#fred.get_series_latest_release',
                                                                                                 'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred.get_series_vintage_dates': ( '00_fred/fred_api.html#fred.get_series_vintage_dates',
                                                                                                'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred.search': ( '00_fred/fred_api.html#fred.search',
                                                                              'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred.search_by_category': ( '00_fred/fred_api.html#fred.search_by_category',
                                                                                          'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred.search_by_release': ( '00_fred/fred_api.html#fred.search_by_release',
                                                                                         'finsets/fred/fred_api.py')},
            'finsets.papers.hassan_etal_2019': { 'finsets.papers.hassan_etal_2019.get_raw_data': ( '02_papers/hassan_etal_2019.html#get_raw_data',
                                                                                                   'finsets/papers/hassan_etal_2019.py'),
                                                 'finsets.papers.hassan_etal_2019.list_all_vars': ( '02_papers/hassan_etal_2019.html#list_all_vars',
                                                                                                    'finsets/papers/hassan_etal_2019.py'),
                                                 'finsets.papers.hassan_etal_2019.process_raw_data': ( '02_papers/hassan_etal_2019.html#process_raw_data',
                                                                                                       'finsets/papers/hassan_etal_2019.py')},
            'finsets.wrds.compa': { 'finsets.wrds.compa.default_raw_vars': ('01_wrds/compa.html#default_raw_vars', 'finsets/wrds/compa.py'),
                                    'finsets.wrds.compa.features': ('01_wrds/compa.html#features', 'finsets/wrds/compa.py'),
                                    'finsets.wrds.compa.get_raw_data': ('01_wrds/compa.html#get_raw_data', 'finsets/wrds/compa.py'),
                                    'finsets.wrds.compa.list_all_vars': ('01_wrds/compa.html#list_all_vars', 'finsets/wrds/compa.py'),
                                    'finsets.wrds.compa.parse_varlist': ('01_wrds/compa.html#parse_varlist', 'finsets/wrds/compa.py'),
                                    'finsets.wrds.compa.process_raw_data': ( '01_wrds/compa.html#process_raw_data',
                                                                             'finsets/wrds/compa.py')},
            'finsets.wrds.compa_ccm': { 'finsets.wrds.compa_ccm.default_raw_vars': ( '01_wrds/compa_ccm.html#default_raw_vars',
                                                                                     'finsets/wrds/compa_ccm.py'),
                                        'finsets.wrds.compa_ccm.features': ('01_wrds/compa_ccm.html#features', 'finsets/wrds/compa_ccm.py'),
                                        'finsets.wrds.compa_ccm.get_raw_data': ( '01_wrds/compa_ccm.html#get_raw_data',
                                                                                 'finsets/wrds/compa_ccm.py'),
                                        'finsets.wrds.compa_ccm.list_all_vars': ( '01_wrds/compa_ccm.html#list_all_vars',
                                                                                  'finsets/wrds/compa_ccm.py'),
                                        'finsets.wrds.compa_ccm.parse_varlist': ( '01_wrds/compa_ccm.html#parse_varlist',
                                                                                  'finsets/wrds/compa_ccm.py'),
                                        'finsets.wrds.compa_ccm.process_raw_data': ( '01_wrds/compa_ccm.html#process_raw_data',
                                                                                     'finsets/wrds/compa_ccm.py')},
            'finsets.wrds.compq': { 'finsets.wrds.compq.default_raw_vars': ('01_wrds/compq.html#default_raw_vars', 'finsets/wrds/compq.py'),
                                    'finsets.wrds.compq.features': ('01_wrds/compq.html#features', 'finsets/wrds/compq.py'),
                                    'finsets.wrds.compq.get_raw_data': ('01_wrds/compq.html#get_raw_data', 'finsets/wrds/compq.py'),
                                    'finsets.wrds.compq.list_all_vars': ('01_wrds/compq.html#list_all_vars', 'finsets/wrds/compq.py'),
                                    'finsets.wrds.compq.parse_varlist': ('01_wrds/compq.html#parse_varlist', 'finsets/wrds/compq.py'),
                                    'finsets.wrds.compq.process_raw_data': ('01_wrds/compq.html#process_raw_data', 'finsets/wrds/compq.py'),
                                    'finsets.wrds.compq.ytd_to_quarterly': ( '01_wrds/compq.html#ytd_to_quarterly',
                                                                             'finsets/wrds/compq.py')},
            'finsets.wrds.compq_ccm': { 'finsets.wrds.compq_ccm.default_raw_vars': ( '01_wrds/compq_ccm.html#default_raw_vars',
                                                                                     'finsets/wrds/compq_ccm.py'),
                                        'finsets.wrds.compq_ccm.features': ('01_wrds/compq_ccm.html#features', 'finsets/wrds/compq_ccm.py'),
                                        'finsets.wrds.compq_ccm.get_raw_data': ( '01_wrds/compq_ccm.html#get_raw_data',
                                                                                 'finsets/wrds/compq_ccm.py'),
                                        'finsets.wrds.compq_ccm.list_all_vars': ( '01_wrds/compq_ccm.html#list_all_vars',
                                                                                  'finsets/wrds/compq_ccm.py'),
                                        'finsets.wrds.compq_ccm.parse_varlist': ( '01_wrds/compq_ccm.html#parse_varlist',
                                                                                  'finsets/wrds/compq_ccm.py'),
                                        'finsets.wrds.compq_ccm.process_raw_data': ( '01_wrds/compq_ccm.html#process_raw_data',
                                                                                     'finsets/wrds/compq_ccm.py'),
                                        'finsets.wrds.compq_ccm.ytd_to_quarterly': ( '01_wrds/compq_ccm.html#ytd_to_quarterly',
                                                                                     'finsets/wrds/compq_ccm.py')},
            'finsets.wrds.crspd': { 'finsets.wrds.crspd.default_raw_vars': ('01_wrds/crspd.html#default_raw_vars', 'finsets/wrds/crspd.py'),
                                    'finsets.wrds.crspd.delist_adj_ret': ('01_wrds/crspd.html#delist_adj_ret', 'finsets/wrds/crspd.py'),
                                    'finsets.wrds.crspd.features': ('01_wrds/crspd.html#features', 'finsets/wrds/crspd.py'),
                                    'finsets.wrds.crspd.get_raw_data': ('01_wrds/crspd.html#get_raw_data', 'finsets/wrds/crspd.py'),
                                    'finsets.wrds.crspd.list_all_vars': ('01_wrds/crspd.html#list_all_vars', 'finsets/wrds/crspd.py'),
                                    'finsets.wrds.crspd.parse_varlist': ('01_wrds/crspd.html#parse_varlist', 'finsets/wrds/crspd.py'),
                                    'finsets.wrds.crspd.process_raw_data': ( '01_wrds/crspd.html#process_raw_data',
                                                                             'finsets/wrds/crspd.py')},
            'finsets.wrds.crspm': { 'finsets.wrds.crspm.default_raw_vars': ('01_wrds/crspm.html#default_raw_vars', 'finsets/wrds/crspm.py'),
                                    'finsets.wrds.crspm.delist_adj_ret': ('01_wrds/crspm.html#delist_adj_ret', 'finsets/wrds/crspm.py'),
                                    'finsets.wrds.crspm.features': ('01_wrds/crspm.html#features', 'finsets/wrds/crspm.py'),
                                    'finsets.wrds.crspm.get_raw_data': ('01_wrds/crspm.html#get_raw_data', 'finsets/wrds/crspm.py'),
                                    'finsets.wrds.crspm.list_all_vars': ('01_wrds/crspm.html#list_all_vars', 'finsets/wrds/crspm.py'),
                                    'finsets.wrds.crspm.parse_varlist': ('01_wrds/crspm.html#parse_varlist', 'finsets/wrds/crspm.py'),
                                    'finsets.wrds.crspm.process_raw_data': ( '01_wrds/crspm.html#process_raw_data',
                                                                             'finsets/wrds/crspm.py')},
            'finsets.wrds.ibes_ltg': { 'finsets.wrds.ibes_ltg.default_raw_vars': ( '01_wrds/ibes_ltg.html#default_raw_vars',
                                                                                   'finsets/wrds/ibes_ltg.py'),
                                       'finsets.wrds.ibes_ltg.get_raw_data': ( '01_wrds/ibes_ltg.html#get_raw_data',
                                                                               'finsets/wrds/ibes_ltg.py'),
                                       'finsets.wrds.ibes_ltg.list_all_vars': ( '01_wrds/ibes_ltg.html#list_all_vars',
                                                                                'finsets/wrds/ibes_ltg.py'),
                                       'finsets.wrds.ibes_ltg.parse_varlist': ( '01_wrds/ibes_ltg.html#parse_varlist',
                                                                                'finsets/wrds/ibes_ltg.py')},
            'finsets.wrds.linking': { 'finsets.wrds.linking.gvkey_permno_a': ( '01_wrds/linking.html#gvkey_permno_a',
                                                                               'finsets/wrds/linking.py'),
                                      'finsets.wrds.linking.gvkey_permno_m': ( '01_wrds/linking.html#gvkey_permno_m',
                                                                               'finsets/wrds/linking.py'),
                                      'finsets.wrds.linking.gvkey_permno_q': ( '01_wrds/linking.html#gvkey_permno_q',
                                                                               'finsets/wrds/linking.py'),
                                      'finsets.wrds.linking.ibes_ticker_permno': ( '01_wrds/linking.html#ibes_ticker_permno',
                                                                                   'finsets/wrds/linking.py')},
            'finsets.wrds.ratios': { 'finsets.wrds.ratios.get_raw_data': ('01_wrds/ratios.html#get_raw_data', 'finsets/wrds/ratios.py'),
                                     'finsets.wrds.ratios.keep_only_ratios': ( '01_wrds/ratios.html#keep_only_ratios',
                                                                               'finsets/wrds/ratios.py'),
                                     'finsets.wrds.ratios.list_all_vars': ('01_wrds/ratios.html#list_all_vars', 'finsets/wrds/ratios.py'),
                                     'finsets.wrds.ratios.process_raw_data': ( '01_wrds/ratios.html#process_raw_data',
                                                                               'finsets/wrds/ratios.py')},
            'finsets.wrds.wrds_api': { 'finsets.wrds.wrds_api.Connection': ('01_wrds/wrds_api.html#connection', 'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.__check_schema_perms': ( '01_wrds/wrds_api.html#connection.__check_schema_perms',
                                                                                                  'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.__create_pgpass_file_unix': ( '01_wrds/wrds_api.html#connection.__create_pgpass_file_unix',
                                                                                                       'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.__create_pgpass_file_win32': ( '01_wrds/wrds_api.html#connection.__create_pgpass_file_win32',
                                                                                                        'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.__enter__': ( '01_wrds/wrds_api.html#connection.__enter__',
                                                                                       'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.__exit__': ( '01_wrds/wrds_api.html#connection.__exit__',
                                                                                      'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.__get_schema_for_view': ( '01_wrds/wrds_api.html#connection.__get_schema_for_view',
                                                                                                   'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.__get_user_credentials': ( '01_wrds/wrds_api.html#connection.__get_user_credentials',
                                                                                                    'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.__init__': ( '01_wrds/wrds_api.html#connection.__init__',
                                                                                      'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.__make_sa_engine_conn': ( '01_wrds/wrds_api.html#connection.__make_sa_engine_conn',
                                                                                                   'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.__write_pgpass_file': ( '01_wrds/wrds_api.html#connection.__write_pgpass_file',
                                                                                                 'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.close': ( '01_wrds/wrds_api.html#connection.close',
                                                                                   'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.connect': ( '01_wrds/wrds_api.html#connection.connect',
                                                                                     'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.create_pgpass_file': ( '01_wrds/wrds_api.html#connection.create_pgpass_file',
                                                                                                'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.describe_table': ( '01_wrds/wrds_api.html#connection.describe_table',
                                                                                            'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.get_row_count': ( '01_wrds/wrds_api.html#connection.get_row_count',
                                                                                           'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.get_table': ( '01_wrds/wrds_api.html#connection.get_table',
                                                                                       'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.list_libraries': ( '01_wrds/wrds_api.html#connection.list_libraries',
                                                                                            'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.list_tables': ( '01_wrds/wrds_api.html#connection.list_tables',
                                                                                         'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.load_library_list': ( '01_wrds/wrds_api.html#connection.load_library_list',
                                                                                               'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.raw_sql': ( '01_wrds/wrds_api.html#connection.raw_sql',
                                                                                     'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.NotSubscribedError': ( '01_wrds/wrds_api.html#notsubscribederror',
                                                                                     'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.SchemaNotFoundError': ( '01_wrds/wrds_api.html#schemanotfounderror',
                                                                                      'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.download': ('01_wrds/wrds_api.html#download', 'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.validate_dates': ( '01_wrds/wrds_api.html#validate_dates',
                                                                                 'finsets/wrds/wrds_api.py')}}}
